@@ -1,6 +1,6 @@
 class aml_test:
     def __init__(self):
-        from dask_cloudprovider import AzureMLCluster
+        from dask_cloudprovider import AzureMLCluster  # noqa
 
 
 if __name__ == "__main__":
